#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/sem.h>

int main(void) {
    int child_pid;
    int sem_key1=12345;
    int sem_key2=54321;
    int returncode_semget1, returncode_semget2, returncode_semctl;
    int output;

    /* Disable buffering for stdout */
    setbuf(stdout, NULL);

    /* Create new set of semaphores 12345 with one semaphore.
     * IPC_CREATE = Create semaphore if it does not exist
     * IPC_EXCL   = Create new set of semaphores instead of using a potentially existing one */
    returncode_semget1 = semget(sem_key1, 1, IPC_CREAT | IPC_EXCL | 0600);
    if (returncode_semget1 < 0) {
        printf("Error while creating set of semaphores %i\n", sem_key1);
        perror("semget");
        exit(1);
    }
    /* Create new set of semaphores 54321 with one semaphore. */
    returncode_semget2 = semget(sem_key2, 1, IPC_CREAT | IPC_EXCL | 0600);
    if (returncode_semget2 < 0) {
        printf("Error while creating set of semaphores %i\n", sem_key2);
        perror("semget");
        exit(1);
    }
    /* Define P and V operation as increasing resp. decreasing the semaphore value by one */
    struct sembuf p_operation = {0, -1, 0};
    struct sembuf v_operation = {0, 1, 0};

    /* Set first semaphore of set 12345 initially to 1 */
    returncode_semctl = semctl(returncode_semget1, 0, SETVAL, 1);
    /* Set first semaphore of set 54321 initially to 0 */
    returncode_semctl = semctl(returncode_semget2, 0, SETVAL, 0);

    output = semctl(returncode_semget1, 0, GETVAL, 0);
    printf("Value of semaphore with ID %i and key %i: %i\n", returncode_semget1, sem_key1, output);
    output = semctl(returncode_semget2, 0, GETVAL, 0);
    printf("Value of semaphore with ID %i and key %i: %i\n", returncode_semget2, sem_key2, output);

    /* Create child process */
    child_pid = fork();

    /* child process */
    if (child_pid == 0) {
        for (int i=0;i<5;i++) {
            semop(returncode_semget2, &p_operation, 1);
            /* Begin: cricitical section */
            printf("B");
            sleep(1);
            /* End: cricitical section */
            semop(returncode_semget1, &v_operation, 1);
        }
        exit(0);
    }

    /* Parent process */
    if (child_pid > 0) {
        for (int i=0;i<5;i++) {
            semop(returncode_semget1, &p_operation, 1);
            /* Begin: cricitical section */
            printf("A");
            sleep(1);
            /* End: cricitical section */
            semop(returncode_semget2, &v_operation, 1);
        }
    }

    /* Wait for the child process to terminate */
    wait(NULL);
    printf("\n");

    /* Remove semaphore sets */
    returncode_semctl = semctl(returncode_semget1, 0, IPC_RMID, 0);
    if (returncode_semctl < 0) {
        printf("Error while creating set of semaphores %i\n", sem_key1);
        exit(1);
    }
    else {
        printf("Set of semaphores with ID %i and key %i removed\n", returncode_semget1, sem_key1);
    }
    returncode_semctl = semctl(returncode_semget2, 0, IPC_RMID, 0);
    if (returncode_semctl < 0) {
        printf("Error while creating set of semaphores %i\n", sem_key2);
        exit(1);
    }
    else {
        printf("Set of semaphores with ID %i and key %i removed\n", returncode_semget2, sem_key2);
    }
    exit(0);
}
