#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <assert.h>
#include <sys/mman.h>
#include <sys/stat.h>        /* For mode constants */
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "shared.h"

/* Warten von 1 bis 10000 Millisekunden (10 Sekunden) */
void msleep(unsigned int ms)
{
	struct timespec ts;

	assert(ms > 0);
	assert(ms <= 10000);

	ts.tv_sec = ms / 1000;
	ts.tv_nsec = (ms * 1000000) % 1000000000;
	nanosleep(&ts, NULL);
}

int main(int argc, char *argv[])
{
    if (argc != 5)
    {
        fprintf(stderr, "Usage: %s <SHM name> <symbol> <length> <wait time>\n", argv[0]);
        return EXIT_FAILURE;
    }

    char symbol = argv[2][0];
    ssize_t length = atoi(argv[3]);
    if ((length < 0) || (length > 40))
    {
        fprintf(stderr, "LENGTH must be between 0 and 40.\n");
        return EXIT_FAILURE;
    }

    int duration = atoi(argv[4]);

    int shmFd = shm_open(argv[1], O_RDWR, S_IRUSR | S_IWUSR);
    if (shmFd < 0)
    {
        perror("shm_open");
        return EXIT_FAILURE;
    }
    pid_t mypid = getpid();
    struct shared *mybuffer;
    mybuffer = mmap(NULL, sizeof(struct shared), PROT_READ | PROT_WRITE, MAP_SHARED, shmFd, 0);

    for (int i = 0; i < 10; i++)
    {
        mybuffer->pid = mypid;
        mybuffer->i = i;
        mybuffer->length = length;
        memset(&(mybuffer->name), symbol, length);
        msleep(duration);
    }

	return EXIT_SUCCESS;
}
