#include <assert.h>
#include <errno.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>

#include "net/sock/udp.h"
#include "test_utils/expect.h"
#include "ztimer.h"

#include "constants.h"
#include "stack.h"

#define _TEST_BUFFER_SIZE   (128)

static sock_udp_t _sock;

static void sock_udp_create4_full(void)
{
    static const sock_udp_ep_t local = { .family = AF_INET, .netif = _TEST_NETIF,
                                         .port = _TEST_PORT_LOCAL };
    static const sock_udp_ep_t remote = { .family = AF_INET,
                                          .port = _TEST_PORT_REMOTE,
                                          .addr =  { .ipv4_u32 = _TEST_ADDR4_REMOTE } };
    sock_udp_ep_t ep;

    assert(0 == sock_udp_create(&_sock, &local, &remote, SOCK_FLAGS_REUSE_EP));
    assert(0 == sock_udp_get_local(&_sock, &ep));
    assert(0 == sock_udp_get_remote(&_sock, &ep));
}


static void _sock_udp_recv4(void)
{
    static const sock_udp_ep_t local = { .family = AF_INET,
                                          .port = _TEST_PORT_LOCAL };
    static const sock_udp_ep_t remote = { .addr = { .ipv4_u32 = _TEST_ADDR4_REMOTE },
                                          .family = AF_INET,
                                          .port = _TEST_PORT_REMOTE };
    void *data = NULL, *ctx = NULL;

    sock_udp_create(&_sock, &local, &remote, SOCK_FLAGS_REUSE_EP);
    sock_udp_recv_buf(&_sock, &data, &ctx, SOCK_NO_TIMEOUT, NULL);
    sock_udp_recv_buf(&_sock, &data, &ctx, SOCK_NO_TIMEOUT, NULL);
}

static void _sock_udp_send4(void)
{
    static const sock_udp_ep_t local = { .addr = { .ipv4_u32 = _TEST_ADDR4_LOCAL },
                                         .family = AF_INET,
                                         .netif = _TEST_NETIF,
                                         .port = _TEST_PORT_LOCAL };
    static const sock_udp_ep_t remote = { .addr = { .ipv4_u32 = _TEST_ADDR4_REMOTE },
                                          .family = AF_INET,
                                          .port = _TEST_PORT_REMOTE };

    sock_udp_create(&_sock, &local, &remote, SOCK_FLAGS_REUSE_EP);
    sock_udp_send(&_sock, "ABCD", sizeof("ABCD"), NULL);
}

int main(void)
{
    _net_init();
    sock_udp_create4_full();
    _sock_udp_recv4();
    _sock_udp_send4();

    sock_udp_close(&_sock);
    memset(&_sock, 0, sizeof(_sock));

    return 0;
}
